# -*- coding: utf-8 -*-

# __init__.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Gracie - OpenID provider for local accounts

Gracie is an OpenID server (a "provider" in OpenID terminology) that
serves OpenID identities for the local system PAM accounts. It
authenticates users with a username/password challenge.

The OpenID protocol is documented at <URL:http://openid.net/>.
"""

from server import __version__

__date__ = "2007-03-28"
__author_name__ = "Ben Finney"
__author_email__ = "ben+python@benfinney.id.au"
__author__ = "%s <%s>" % (__author_name__, __author_email__)
__copyright__ = "Copyright © %s %s" % (
    __date__.split('-')[0], __author_name__
)
__license__ = "GPL"
__url__ = "http://trac.whitetree.org/gracie/"



# -*- coding: utf-8 -*-

# httpserver.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Behaviour for HTTP server portion of Gracie
"""

import logging
import urlparse
from BaseHTTPServer import HTTPServer as BaseHTTPServer

import server

# Get the Python logging instance for this module
_logger = logging.getLogger("gracie.httpserver")

default_host = "localhost"
default_port = 8000


class BaseHTTPServer(BaseHTTPServer, object):
    """ Shim to insert base object type into hierarchy """


http_port = 80

def net_location(host, port=None):
    """ Construct a location string from host string and port number """
    if port is None or port == http_port:
        location_spec = "%(host)s"
    else:
        location_spec = "%(host)s:%(port)s"
    location = location_spec % locals()
    return location

default_root_url = urlparse.urlunsplit(
    ("http", net_location(default_host, default_port), "/", "", "")
)


class HTTPServer(BaseHTTPServer):
    """ Server for HTTP protocol requests """

    def __init__(self,
            server_address, RequestHandlerClass, gracie_server
    ):
        """ Set up a new instance """
        self.gracie_server = gracie_server
        self._setup_version()
        self._setup_logging()
        super(HTTPServer, self).__init__(
            server_address, RequestHandlerClass
        )
        self._log_location()

    def _setup_version(self):
        """ Set up version string for this server """
        self.version = self.gracie_server.version

    def _setup_logging(self):
        """ Set up logging for this server """
        version = self.version
        _logger.info(
            "Starting Gracie HTTP server (version %(version)s)"
            % locals()
        )

    def server_bind(self):
        """ Bind and name the server """
        super(HTTPServer, self).server_bind()
        self.server_location = net_location(
            self.server_name, self.server_port
        )

    def _log_location(self):
        """ Log the net location of the server """
        location = net_location(*self.server_address)
        _logger.info(
            "Listening on address %(location)s"
            % locals()
        )

    def handle_request(self):
        """ Handle a single request """
        try:
            super(HTTPServer, self).handle_request()
        except (KeyboardInterrupt, SystemExit), e:
            exc_name = e.__class__.__name__
            message = "Received %(exc_name)s" % locals()
            _logger.warn(message)
            raise
        except Exception, e:
            message = str(e)
            _logger.error(message)
            raise



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_server.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for server module
"""

import sys
import os
from StringIO import StringIO
import logging
import optparse

import scaffold
from scaffold import Mock
from test_authservice import Stub_AuthService
from test_httpresponse import Stub_ResponseHeader, Stub_Response

from gracie import server


class Test_remove_standard_files(scaffold.TestCase):
    """ Test cases for remove_standard_files function """

    def setUp(self):
        """ Set up test fixtures """
        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.server_stdin_prev = server.sys.stdin
        server.sys.stdin = Mock('sys.stdin')
        self.server_stdout_prev = server.sys.stdout
        server.sys.stdout = Mock('sys.stdout')
        self.server_stderr_prev = server.sys.stderr
        server.sys.stderr = Mock('sys.stderr')

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        server.sys.stdin = self.server_stdin_prev
        server.sys.stdout = self.server_stdout_prev
        server.sys.stderr = self.server_stderr_prev


class Mock_fork(object):
    """ Mock callable for os.fork() """

    def __init__(self, name, pids):
        self.mock_name = name
        self._return_pids = iter(pids)

    def _make_message(self, args, kwargs):
        parts = [repr(a) for a in args]
        parts.extend(
            '%s=%r' % (items) for items in sorted(kwargs.items()))
        msg = 'Called %s(%s)' % (self.mock_name, ', '.join(parts))
        if len(msg) > 80:
            msg = 'Called %s(\n    %s)' % (
                self.mock_name, ',\n    '.join(parts))
        return msg

    def __call__(self, *args, **kwargs):
        print self._make_message(args, kwargs)
        return self._return_pids.next()

class Test_become_daemon(scaffold.TestCase):
    """ Test cases for become_daemon function """

    def setUp(self):
        """ Set up test fixtures """
        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.fork_prev = server.os.fork
        test_pids = [0, 0]
        server.os.fork = Mock_fork('os.fork', test_pids)
        self.setsid_prev = server.os.setsid
        server.os.setsid = Mock('os.setsid')
        self._exit_prev = server.os._exit
        server.os._exit = Mock('os._exit')
        self.exit_prev = server.sys.exit
        server.sys.exit = Mock('sys.exit')
        self.remove_files_prev = server.remove_standard_files
        server.remove_standard_files = Mock('remove_standard_files')

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        server.os.fork = self.fork_prev
        server.os.setsid = self.setsid_prev
        server.os._exit = self._exit_prev
        server.sys.exit = self.exit_prev
        server.remove_standard_files = self.remove_files_prev

    def test_parent_exits(self):
        """ become_daemon parent process should exit """
        parent_pid = 23
        server.os.fork = Mock_fork('os.fork', [parent_pid])
        expect_stdout = """\
            Called os.fork()
            Called os._exit(0)
            ...
            """
        server.become_daemon()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_child_starts_new_process_group(self):
        """ become_daemon child should start new process group """
        expect_stdout = """\
            Called os.fork()
            Called os.setsid()
            ...
            """
        server.become_daemon()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_child_forks_next_parent_exits(self):
        """ become_daemon should fork, then exit if parent """
        test_pids = [0, 42]
        server.os.fork = Mock_fork('os.fork', test_pids)
        expect_stdout = """\
            Called os.fork()
            Called os.setsid()
            Called os.fork()
            Called os._exit(0)
            ...
            """
        server.become_daemon()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_child_forks_next_child_continues(self):
        """ become_daemon should fork, then continue if child """
        expect_stdout = """\
            Called os.fork()
            Called os.setsid()
            Called os.fork()
            Called remove_standard_files()
            """
        server.become_daemon()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_removes_standard_files(self):
        """ become_daemon should request removal of standard files """
        expect_stdout = """\
            ...
            Called remove_standard_files()
            """
        server.become_daemon()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )


def stub_server_bind(server):
    """ Stub method to get server location """
    (host, port) = server.server_address
    (server.server_name, server.server_port) = (host, port)

class Stub_HTTPServer(object):
    """ Stub class for HTTPServer """
    def __init__(self,
            server_address, RequestHandlerClass, gracie_server
    ):
        """ Set up a new instance """

    server_bind = stub_server_bind


class Stub_ConsumerAuthStore(object):
    """ Stub class for ConsumerAuthStore """

    def __init__(self):
        self._authorisations = dict()

    def store_authorisation(self, auth_tuple, status):
        self._authorisations[auth_tuple] = status

    def is_authorised(self, auth_tuple):
        return self._authorisations.get(auth_tuple, False)

class Stub_ConsumerAuthStore_always_auth(Stub_ConsumerAuthStore):
    """ ConsumerAuthStore stub that always authorises """

    def is_authorised(self, auth_tuple):
        return True

class Stub_ConsumerAuthStore_never_auth(Stub_ConsumerAuthStore):
    """ ConsumerAuthStore stub that never authorises """

    def is_authorised(self, auth_tuple):
        return False


class Stub_SessionManager(object):
    """ Stub class for SessionManager """

    def store_session(self, session):
        pass

    def get_session(self, session_id):
        return None

    def remove_session(self, session_id):
        pass


class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """


class Stub_OpenIDError(Exception):
    """ Stub error class for openid module """

class Stub_OpenIDStore(object):
    """ Stub class for openid backing store """

    def __init__(self, _, *args, **kwargs):
        """ Set up a new instance """

class Stub_OpenIDServer(object):
    """ Stub class for an OpenID protocol server """

    def __init__(self, store):
        """ Set up a new instance """

    def decodeRequest(self, request):
        return Stub_OpenIDResponse()

    def encodeResponse(self, response):
        return Stub_OpenIDWebResponse()

class Stub_OpenIDRequest(object):
    """ Stub class for an OpenID protocol request """

    def __init__(self, http_query, params=None):
        """ Set up a new instance """

        self.mode = http_query.get('openid.mode')
        keys = [
            'identity', 'trust_root', 'immediate', 'return_to',
        ]
        if params is None:
            params = dict()
        for key in keys:
            setattr(self, key, None)
            if key in params:
                setattr(self, key, params[key])

    def answer(self, allow, server_url=None):
        response = Stub_OpenIDResponse(dict(
            allow = allow,
            server_url = server_url,
        ))
        return response

class Stub_OpenIDResponse(object):
    """ Stub class for an OpenID protocol response """

    def __init__(self, params=None):
        self.params = params

    def encodeToURL(self):
        url = "http://stub/openid_response/" + ";".join([
            "%s=%s" % (key, val) for key, val in self.params.items()
        ])
        return url

class Stub_OpenIDWebResponse(object):
    """ Stub class for an encoded OpenID response """

    def __init__(self):
        """ Set up a new instance """
        self.code = 200
        self.headers = {"openid": "yes"}
        self.body = "OpenID response"

def make_default_opts():
    """ Create commandline opts instance with required values """
    opts = optparse.Values(dict(
        datadir = "/tmp",
        host = "example.org", port = 9779,
    ))
    return opts


class Test_GracieServer(scaffold.TestCase):
    """ Test cases for GracieServer class """

    def setUp(self):
        """ Set up test fixtures """

        self.server_class = server.GracieServer

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.openid_server_prev = server.OpenIDServer
        self.openid_store_prev = server.OpenIDStore
        self.consumer_store_prev = server.ConsumerAuthStore
        self.session_manager_prev = server.SessionManager
        server.OpenIDServer = Stub_OpenIDServer
        server.OpenIDStore = Stub_OpenIDStore
        server.ConsumerAuthStore = Stub_ConsumerAuthStore
        server.SessionManager = Stub_SessionManager

        self.httpserver_class_prev = server.HTTPServer
        server.HTTPServer = Stub_HTTPServer
        self.handler_class_prev = server.HTTPRequestHandler
        server.RequestHandlerClass = Stub_HTTPRequestHandler
        self.default_port_prev = server.default_port
        server.default_port = 7654

        self.valid_servers = {
            'simple': dict(
            ),
            'with-opts': dict(
                opts = dict(
                    foo="spam",
                    bar="eggs",
                ),
            ),
            'datadir': dict(
                opts = dict(
                    datadir = "/foo/bar",
                ),
                datadir = "/foo/bar",
            ),
        }

        for key, params in self.valid_servers.items():
            args = params.get('args')
            if not args:
                args = dict()
            opts = make_default_opts()
            opts._update_loose(params.get('opts', dict()))
            params['opts'] = opts
            args.update(dict(
                socket_params=None,
                opts=opts,
            ))
            instance = self.server_class(**args)
            params['args'] = args
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_servers
        )

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        server.HTTPServer = self.httpserver_class_prev
        server.HTTPRequestHandler = self.handler_class_prev
        server.default_port = self.default_port_prev
        server.OpenIDServer = self.openid_server_prev
        server.OpenIDStore = self.openid_store_prev
        server.ConsumerAuthStore = self.consumer_store_prev
        server.SessionManager = self.session_manager_prev

    def test_instantiate(self):
        """ New GracieServer instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_version_as_specified(self):
        """ GracieServer should have specified version string """
        params = self.valid_servers['simple']
        version_prev = server.__version__
        version_test = "1.414.test"
        server.__version__ = version_test
        instance = self.server_class(**params['args'])
        self.failUnlessEqual(version_test, instance.version)
        server.__version__ = version_prev

    def test_opts_as_specified(self):
        """ GracieServer should have specified opts mapping """
        params = self.valid_servers['with-opts']
        instance = params['instance']
        opts = params['opts']
        self.failUnlessEqual(opts, instance.opts)

    def test_server_creates_http_server(self):
        """ GracieServer should create an HTTP server """
        params = self.valid_servers['simple']
        args = params['args']
        opts = params['opts']
        server_address = (opts.host, opts.port)
        http_server_class_prev = server.HTTPServer
        server.HTTPServer = Mock('HTTPServer_class')
        expect_stdout = """\
            Called HTTPServer_class(
                %(server_address)r,
                <class '...HTTPRequestHandler'>,
                <gracie.server.GracieServer object ...>)
            """ % locals()
        instance = self.server_class(**params['args'])
        server.HTTPServer = http_server_class_prev
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_server_has_openid_server(self):
        """ GracieServer should have an openid_server attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        openid_server = instance.openid_server
        self.failUnless(isinstance(openid_server, Stub_OpenIDServer))

    def test_openid_store_created_with_datadir(self):
        """ OpenIDStore should be created with specified datadir """
        params = self.valid_servers['datadir']
        datadir = params['datadir']
        openid_store_class_prev = server.OpenIDStore
        server.OpenIDStore = Mock('FileOpenIDStore_class')
        expect_stdout = """\
            Called FileOpenIDStore_class(%(datadir)r)
            """ % locals()
        instance = self.server_class(**params['args'])
        server.OpenIDStore = openid_store_class_prev
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_server_has_auth_service(self):
        """ GracieServer should have an auth_service attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        auth_service = instance.auth_service
        self.failUnless(auth_service is not None)

    def test_server_has_session_manager(self):
        """ GracieServer should have a sess_manager attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        sess_manager = instance.sess_manager
        self.failUnless(sess_manager is not None)

    def test_server_has_authorisation_store(self):
        """ GracieServer should have a consumer_auth_store attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        consumer_auth_store = instance.consumer_auth_store
        self.failUnless(consumer_auth_store is not None)

    def test_serve_forever_is_callable(self):
        """ GracieServer.serve_forever should be callable """
        self.failUnless(callable(self.server_class.serve_forever))


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



# -*- coding: utf-8 -*-

# httprequest.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" HTTP request handling
"""

from BaseHTTPServer import BaseHTTPRequestHandler
import logging
import time
import cgi
import Cookie
import urlparse
import routes
from openid.server.server import BROWSER_REQUEST_MODES

from gracie import pagetemplate
from gracie.httpresponse import ResponseHeader, Response
from gracie.httpresponse import response_codes as http_codes
from gracie.authservice import AuthenticationError

session_cookie_name = "gracie_session"

_logger = logging.getLogger("gracie.httprequest")


class BaseHTTPRequestHandler(BaseHTTPRequestHandler, object):
    """ Shim to insert base object type into hierarchy """


mapper = routes.Mapper()
mapper.connect('root', '', controller='about', action='view')
mapper.connect('openid', 'openidserver', controller='openid')
mapper.connect('identity', 'id/:name',
               controller='identity', action='view')
mapper.connect('login', 'login', controller='login', action='view')
mapper.connect('logout', 'logout', controller='logout', action='view')

class HTTPRequestHandler(BaseHTTPRequestHandler):
    """ Handler for individual HTTP requests """

    def __init__(self, request, client_address, server):
        """ Set up a new instance """
        self.gracie_server = server.gracie_server
        self._setup_version()
        super(HTTPRequestHandler, self).__init__(
            request, client_address, server
        )

    def _setup_version(self):
        """ Set up the version string """
        version = self.gracie_server.version
        self.server_version = "Gracie/%(version)s" % locals()

    def log_message(self, format, *args, **kwargs):
        """ Log a message via the server's logger """
        logger = _logger
        loglevel = logging.INFO
        logger.log(loglevel, format, *args, **kwargs)

    def _make_server_url(self, path):
        """ Construct a URL to a path on this server """
        root_url = self.gracie_server.opts.root_url
        url = urlparse.urljoin(root_url, path)
        return url

    def _begin_new_session(self):
        """ Begin a new server session """
        sess_manager = self.gracie_server.sess_manager
        self.session = dict()
        session_id = sess_manager.create_session(self.session)
        self.session['session_id'] = session_id

    def _authenticate_session(self, username):
        """ Authenticate the current session as specified username """
        if username is not None:
            auth_entry = self.gracie_server.auth_service.get_entry(username)
            self.session.update(dict(
                username = username,
                auth_entry = auth_entry,
            ))
            _logger.info(
                "Session authenticated as %(username)r" % locals()
            )
        else:
            _logger.info("Session not authenticated")

    def _setup_auth_session(self):
        """ Set up the authentication session """
        sess_manager = self.gracie_server.sess_manager
        session_id = self._get_auth_cookie()
        try:
            self.session = sess_manager.get_session(session_id)
        except KeyError:
            self._begin_new_session()

        username = self.session.get('username')
        self._authenticate_session(username)

    def _remove_auth_session(self):
        """ Remove the authentication session """
        sess_manager = self.gracie_server.sess_manager
        if self.session:
            sess_manager.remove_session(self.session.get('session_id'))
        self._begin_new_session()

        _logger.info("Removed authentication session")

    def _get_cookie(self, name):
        """ Get a cookie from the request """
        value = None
        cookie_string = ";".join(self.headers.getheaders('Cookie'))
        cookies = Cookie.BaseCookie(cookie_string)
        if cookies:
            if session_cookie_name in cookies:
                value = cookies[session_cookie_name].value
        return value

    def _get_auth_cookie(self):
        """ Get the authentication cookie from the request """
        session_id = self._get_cookie('session')
        return session_id

    def _set_cookie(self, response, name, value, expire=None):
        """ Set a cookie in the response header """
        field_name = "Set-Cookie"
        field_value = "%(name)s=%(value)s" % locals()
        if expire:
            field_value = (
                "%(field_value)s;Expires=%(expire)s" % locals())
        field = (field_name, field_value)
        response.header.fields.append(field)

    def _set_auth_cookie(self, response):
        """ Set the authentication cookie in the response """
        field_name = "Set-Cookie"
        epoch = time.gmtime(0)
        expire_immediately = time.strftime(
            '%a, %d-%b-%y %H:%M:%S GMT', epoch)

        session_id = self.session['session_id']
        self._set_cookie(response, session_cookie_name, session_id)

    def _get_username_from_identity(self, identity):
        """ Parse a local username from an OpenID URL """
        name = None
        (_, _, path, _, _) = urlparse.urlsplit(identity)
        route_map = mapper.match(path)
        if route_map:
            if route_map.get('controller') == 'identity':
                name = route_map.get('name')
        return name

    def _make_openid_url(self, username):
        """ Generate the OpenID URL for a username """
        path = "id/%(username)s" % locals()
        url = self._make_server_url(path)
        return url

    def _get_session_openid_url(self):
        """ Get the OpenID URL for the current session """
        openid_url = None
        if self.session:
            username = self.session.get('username')
            openid_url = self._make_openid_url(username)
            self.session['openid_url'] = openid_url
        return openid_url

    def _get_session_auth_entry(self):
        auth_entry = None
        if self.session:
            auth_entry = self.session.get('auth_entry')
        return auth_entry

    def _dispatch(self):
        """ Dispatch to the appropriate controller """
        controller_map = {
            None: self._make_url_not_found_error_response,
            'openid': self._handle_openid_request,
            'about': self._make_about_site_view_response,
            'identity': self._make_identity_view_response,
            'logout': self._make_logout_response,
            'login': self._make_login_response,
        }
        controller_name = None
        if self.route_map:
            controller_name = self.route_map['controller']
        controller = controller_map[controller_name]

        _logger.info(
            "Dispatching to controller %(controller_name)r" % locals()
        )

        response = controller()
        self._set_auth_cookie(response)
        self._send_response(response)

    def _send_response(self, response):
        """ Send an HTTP response to the user agent """
        response.send_to_handler(self)

        _logger.info("Sent HTTP response")

    def _parse_path(self):
        """ Parse the request path """
        keys = ('scheme', 'location', 'path', 'query', 'fragment')
        values = urlparse.urlsplit(self.path)
        self.parsed_url = dict(zip(keys, values))
        self.route_map = mapper.match(self.parsed_url['path'])

    def _parse_query(self):
        """ Parse query fields from the request data """
        self.query = {}
        for key, value in cgi.parse_qsl(self.query_data):
            self.query[key] = value

    def handle(self):
        """ Handle the requests """
        try:
            super(HTTPRequestHandler, self).handle()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception, e:
            message = str(e)
            _logger.error(message)
            response = self._make_internal_error_response(message)
            self._send_response(response)
            raise

    def do_GET(self):
        """ Handle a GET request """
        self._setup_auth_session()
        self._parse_path()
        self.query_data = self.parsed_url['query']
        self._parse_query()
        self._dispatch()

    def do_POST(self):
        """ Handle a POST request """
        self._setup_auth_session()
        self.route_map = mapper.match(self.path)
        content_length = int(self.headers['Content-Length'])
        self.query_data = self.rfile.read(content_length)
        self._parse_query()
        self._dispatch()

    def _handle_openid_request(self):
        """ Handle a request to the OpenID server URL """
        openid_server = self.gracie_server.openid_server
        openid_request = openid_server.decodeRequest(self.query)
        if openid_request is None:
            response = self._make_about_site_view_response()
        else:
            _logger.info("Received OpenID protocol request")
            self.session['last_openid_request'] = openid_request
            if openid_request.mode in BROWSER_REQUEST_MODES:
                response = self._handle_openid_browser_request(
                    openid_request
                )
            else:
                openid_response = openid_server.handleRequest(
                    openid_request
                )
                response = self._make_response_from_openid_response(
                    openid_response
                )

        return response

    def _handle_openid_browser_request(self, openid_request):
        """ Handle an OpenID request with user-agent interaction """
        is_session_identity = False
        identity = openid_request.identity
        if self.session:
            openid_url = self._get_session_openid_url()
            if openid_url:
                if openid_url == identity:
                    # Session is authenticated for requested identity
                    is_session_identity = True

        dispatch_map = {
            'checkid_immediate': self._make_checkid_immediate_response,
            'checkid_setup': self._make_checkid_setup_response,
        }
        response = dispatch_map[openid_request.mode](openid_request,
            is_session_identity = is_session_identity,
        )

        return response

    def _make_checkid_immediate_response(self, request,
        is_session_identity
    ):
        """ Make a response for an OpenID checkid_immediate request """
        response = None

        if is_session_identity:
            openid_response = request.answer(True)
        else:
            openid_response = request.answer(False,
                self._make_server_url("openidserver")
            )
        response = self._make_response_from_openid_response(
            openid_response
        )

        return response

    def _make_checkid_setup_response(self, request,
        is_session_identity
    ):
        """ Make a response for an OpenID checkid_setup request """
        response = None

        if is_session_identity:
            openid_response = request.answer(True)
            response = self._make_response_from_openid_response(
                openid_response
            )
        else:
            response = self._make_wrong_authentication_response(
                request.identity
            )

        return response

    def _make_response_from_openid_response(self, openid_response):
        """ Construct a response to a request to the OpenID server """
        _logger.info("Delegating response to OpenID library")
        openid_server = self.gracie_server.openid_server
        web_response = openid_server.encodeResponse(openid_response)
        header = ResponseHeader(web_response.code)
        for name, value in web_response.headers.items():
            field = (name, value)
            header.fields.append(field)
        response = Response(header, web_response.body)

        return response

    def _make_url_from_openid_response(self, openid_response):
        """ Construct a URL corresponding to an OpenID response """
        _logger.info("Getting URL for OpenID response")
        url = openid_response.encodeToURL()
        return url

    def _make_redirect_response(self, url):
        """ Construct a response for a redirect """
        header = ResponseHeader(http_codes['found'])
        header.fields.append(("Location", url))
        data = ""
        response = Response(header, data)
        _logger.info("Redirecting to %(url)r" % locals())
        return response

    def _get_page_data(self, page):
        """ Get the actual data to be used from a page """
        page.values.update(dict(
            server_version = self.server_version,
            server_location = self.server.server_location,
            auth_entry = self._get_session_auth_entry(),
            openid_url = self._get_session_openid_url(),
            root_url = self._make_server_url(""),
            server_url = self._make_server_url("openidserver"),
            login_url = self._make_server_url("login"),
            logout_url = self._make_server_url("logout"),
        ))
        return page.serialise()

    def _make_internal_error_response(self, message):
        """ Construct an Internal Error error response """
        header = ResponseHeader(http_codes['internal_error'])
        page = pagetemplate.internal_error_page(message)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_url_not_found_error_response(self):
        """ Construct a Not Found error response """
        header = ResponseHeader(http_codes['not_found'])
        page = pagetemplate.url_not_found_page(self.path)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_protocol_error_response(self, message):
        """ Construct a Protocol Error error response """
        header = ResponseHeader(http_codes['ok'])
        page = pagetemplate.protocol_error_page(message)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_about_site_view_response(self):
        """ Construct a response for the about-this-site view """
        header = ResponseHeader(http_codes['ok'])
        page = pagetemplate.about_site_view_page()
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_identity_view_response(self):
        """ Construct a response for an identity view """
        name = self.route_map['name']
        try:
            entry = self.gracie_server.auth_service.get_entry(name)
        except KeyError, e:
            entry = None

        if entry is None:
            header = ResponseHeader(http_codes['not_found'])
            page = pagetemplate.identity_user_not_found_page(name)
        else:
            header = ResponseHeader(http_codes['ok'])
            identity_url = self._make_openid_url(name)
            page = pagetemplate.identity_view_user_page(
                entry, identity_url
            )

        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_logout_response(self):
        """ Construct a response for a logout request """
        self._remove_auth_session()

        root_url = self._make_server_url("")
        response = self._make_redirect_response(root_url)
        return response

    def _make_login_response(self):
        """ Construct a response for a login request """
        controller = {
            'GET': self._make_login_view_response,
            'POST': self._make_login_submit_response,
        }[self.command]
        response = controller()
        return response

    def _make_login_view_response(self):
        """ Construct a response for a login view request """
        header = ResponseHeader(http_codes['ok'])
        page = pagetemplate.login_view_page()
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_login_submit_response(self):
        """ Construct a response for a login submit request """
        if 'cancel' in self.query:
            response = self._make_login_cancelled_response()
        elif 'submit' in self.query:
            response = self._make_login_authenticate_response()
        else:
            response = self._make_url_not_found_error_response()
        return response

    def _make_login_cancelled_response(self):
        """ Construct a response for a login cancel request """
        root_url = self._make_server_url("")
        openid_request = self.session.get('last_openid_request')
        if openid_request:
            return_url = self._make_url_from_openid_response(
                openid_request.answer(False)
            )
        else:
            return_url = root_url
        response = self._make_redirect_response(return_url)
        return response

    def _make_login_authenticate_response(self):
        """ Construct a response for a login authenticate request """
        want_username = self.query.get('username')
        password = self.query.get('password')
        credentials = dict(
            username=want_username,
            password=password
        )
        auth_service = self.gracie_server.auth_service
        try:
            username = auth_service.authenticate(credentials)
            authenticated = True
        except AuthenticationError, e:
            authenticated = False
        if authenticated:
            self._authenticate_session(username)
            response = self._make_login_succeeded_response()
        else:
            response = self._make_login_failed_response()
        return response

    def _make_login_failed_response(self):
        """ Construct a response for a failed login request """
        name = self.query.get('username')
        message = "The login details were incorrect."
        header = ResponseHeader(http_codes['ok'])
        page = pagetemplate.login_submit_failed_page(message, name)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_login_succeeded_response(self):
        """ Construct a response for a successful login request """
        root_url = self._make_server_url("")
        openid_request = self.session.get('last_openid_request')
        openid_url = self._get_session_openid_url()
        if openid_request:
            if openid_request.identity == openid_url:
                openid_server = self.server.gracie_server.openid_server
                openid_response = openid_server.signatory.sign(
                    openid_request.answer(True)
                )
                return_url = self._make_url_from_openid_response(
                    openid_response
                )
                response = self._make_redirect_response(return_url)
            else:
                response = self._make_wrong_authentication_response(
                    openid_request.identity
                )
        else:
            response = self._make_redirect_response(root_url)
        return response

    def _make_wrong_authentication_response(self, want_id):
        """ Make a response for action with wrong session auth """
        want_username = self._get_username_from_identity(want_id)
        header = ResponseHeader(http_codes['ok'])
        page = pagetemplate.wrong_authentication_page(
            want_username = want_username,
            want_id_url = want_id
        )
        data = self._get_page_data(page)
        response = Response(header, data)

        return response



# -*- coding: utf-8 -*-

# authservice.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Authentication services interface
"""

import pwd
import PAM
import logging

_logger = logging.getLogger("gracie.auth")

pam_service_name = "gracie"


class AuthenticationError(EnvironmentError):
    """ Raised when an authentication request fails """

    def __init__(self, code, reason):
        """ Set up a new instance """
        EnvironmentError.__init__(self, code, reason)


class BaseAuthService(object):
    """ Abstract interface to an authentication service """

    # Name of the auth service
    name = NotImplemented

    def get_entry(self, value):
        """ Get an entry from the service by key value """
        raise NotImplementedError

    def authenticate(self, credentials):
        """ Verify credentials against authentication service """
        raise NotImplementedError


class PosixAuthService(BaseAuthService):
    """ Interface to POSIX authentication service """

    def _pwd_entry_to_auth_entry(self, pwd_entry):
        """ Construct an auth entry from a pwd entry """
        (name, _, uid, _, comment, _, _) = pwd_entry
        fullname = comment.split(",")[0]
        entry = dict(
            id=uid,
            name=name,
            fullname=fullname,
        )
        return entry

    def get_entry(self, value):
        """ Get an entry from the service by key value """
        pwd_entry = pwd.getpwnam(value)
        entry = self._pwd_entry_to_auth_entry(pwd_entry)
        return entry


class _PamConversation(object):
    """ PAM authentication conversation """

    def __init__(self, password):
        """ Set up a new instance """
        self.password = password

    def __call__(self, auth, query_list, user_data):
        """ Callback from PAM system """
        _logger.info("PAM: received query")

        response_map = {
            PAM.PAM_PROMPT_ECHO_ON: (self.password, 0),
            PAM.PAM_PROMPT_ECHO_OFF: (self.password, 0),
            PAM.PAM_ERROR_MSG: ('', 0),
            PAM.PAM_TEXT_INFO: ('', 0),
        }
        pam_password_prompts = [
            PAM.PAM_PROMPT_ECHO_ON, PAM.PAM_PROMPT_ECHO_OFF,
        ]

        response_list = []
        for query, qtype in query_list:
            _logger.info(
                "PAM query: type %(qtype)r, %(query)r" % locals())
            response = response_map.get(qtype)
            if response is None:
                _logger.warn(
                    "PAM: unknown query type %(qtype)r" % locals())
                response_list = None
                break
            else:
                sanitised_response = response
                if qtype in pam_password_prompts:
                    sanitised_response = ("<password>", 0)
                _logger.info(
                    "PAM response: sending %(sanitised_response)r"
                    % locals()
                )
                response_list.append(response)
        return response_list

class PamAuthService(PosixAuthService):
    """ Interface to PAM authentication service """

    def __init__(self):
        """ Set up a new instance """
        super(PamAuthService, self).__init__()

        self._pam_auth = PAM.pam()

    def _authenticate_creds(self, username, password):
        """ Authenticate credentials against PAM """
        self._pam_auth.start(pam_service_name)
        self._pam_auth.set_item(PAM.PAM_USER, username)
        conversation = _PamConversation(password)
        self._pam_auth.set_item(PAM.PAM_CONV, conversation)

        flags = 0
        flags |= PAM.PAM_SILENT
        flags |= PAM.PAM_DISALLOW_NULL_AUTHTOK
        self._pam_auth.authenticate(flags)
        self._pam_auth.acct_mgmt()

        got_username = self._pam_auth.get_item(PAM.PAM_USER)
        return got_username

    def authenticate(self, credentials):
        """ Verify credentials against authentication service """
        username = credentials['username']
        password = credentials['password']
        _logger.info("Attempting to authenticate credentials"
            " for %(username)r" % locals()
        )
        try:
            got_username = self._authenticate_creds(username, password)
        except PAM.error, e:
            if len(e.args) < 2:
                (reason, code) = (str(e), 0)
            else:
                (reason, code) = e.args
            raise AuthenticationError(code, reason)

        _logger.info("Successful authentication for %(username)r"
            % locals()
        )
        return got_username




#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_httprequest.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for httprequest module
"""

import sys
from StringIO import StringIO
import logging
import urllib

import scaffold
from scaffold import Mock
from test_authservice import Stub_AuthService
from test_httpresponse import Stub_ResponseHeader, Stub_Response
from test_server import (
    Stub_OpenIDStore, Stub_OpenIDServer, Stub_OpenIDError,
    Stub_OpenIDRequest, Stub_OpenIDResponse, Stub_OpenIDWebResponse,
)

from gracie import httprequest


class Stub_Logger(object):
    """ Stub class for Logger """

    def log(self, format, *args, **kwargs):
        """ Log a message """

    error = log
    warn = log
    info = log
    debug = log

class Stub_SessionManager(object):
    """ Stub class for SessionManager """

    def __init__(self):
        """ Set up a new instance """
        self._sessions = dict()

    def create_session(self, session):
        username = session.get('username')
        session_id = session.get('session_id')
        if session_id is None:
            if username is None:
                session_id = "DEADBEEF"
            else:
                session_id = "DEADBEEF-%(username)s" % session
        session.update(dict(session_id = session_id))
        self._sessions[session_id] = session
        return session_id

    def get_session(self, session_id):
        return self._sessions[session_id]

    def remove_session(self, session_id):
        del self._sessions[session_id]

class Stub_HTTPServer(object):
    """ Stub class for HTTPServer """

    def __init__(self, server_address, handler_class, gracie_server):
        """ Set up a new instance """
        self.gracie_server = gracie_server
        (host, port) = server_address
        self.server_location = "%(host)s:%(port)s" % locals()

class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """

class Stub_OptionValues(object):
    """ Stub class for optparse.OptionValues """

    def __init__(self, opt_dict):
        """ Set up a new instance """
        for key, value in opt_dict.items():
            setattr(self, key, value)

class Stub_GracieServer(object):
    """ Stub class for GracieServer """

    version = "3.14.test"

    def __init__(self, opts):
        """ Set up a new instance """
        self.opts = opts
        server_address = (opts.host, opts.port)
        self.server_location = "%s:%s" % server_address
        self.http_server = Stub_HTTPServer(
            server_address, Stub_HTTPRequestHandler(), self
        )
        store = Stub_OpenIDStore(None)
        self.openid_server = Stub_OpenIDServer(store)
        self.auth_service = Stub_AuthService()
        self.sess_manager = Stub_SessionManager()


class Stub_TCPConnection(object):
    """ Stub class for TCP connection """

    def __init__(self, text):
        """ Set up a new instance """
        self._text = text

    def makefile(self, mode, bufsize):
        """ Make a file handle to the connection stream """
        conn_file = None
        if mode.startswith('r'):
            conn_file = StringIO(self._text)
            conn_file.seek(0)
        elif mode.startswith('w'):
            conn_file = StringIO("")
        return conn_file

class Stub_Request(object):
    """ Stub class for HTTP request encapsulation """

    def __init__(self, method, path,
        version="HTTP/1.1", header=None, query=None,
    ):
        """ Set up a new instance """
        self.method = method.upper()
        self.path = path
        self.version = version
        if header is None:
            header = []
        self.header = header
        if query is None:
            query = dict()
        self.query = query
        self.data = ""

    def _encode_query(self):
        if self.query:
            query_text = urllib.urlencode(self.query)

            if self.method in ["POST", "PUT"]:
                self.header.append(
                    ("Content-Length", str(len(query_text)))
                )
                self.data += query_text
            elif self.method in ["GET", "HEAD"]:
                path = self.path
                self.path = "%(path)s?%(query_text)s" % locals()

    def __str__(self):
        self._encode_query()
        command_text = "%(method)s %(path)s %(version)s" % self.__dict__
        header_text = "\n".join(["%s: %s" % (name, val)
                                 for name, val in self.header])

        lines = []
        lines.append(command_text)
        if header_text:
            lines.append(header_text)
        lines.append("")
        if self.data:
            lines.append(self.data)

        text = "\n".join(lines)
        return text

    def connection(self):
        """ Return a TCP connection to this request """
        return Stub_TCPConnection(str(self))


class Test_HTTPRequestHandler(scaffold.TestCase):
    """ Test cases for HTTPRequestHandler class """

    def setUp(self):
        """ Set up test fixtures """
        self.handler_class = httprequest.HTTPRequestHandler

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.response_class_prev = httprequest.Response
        self.response_header_class_prev = httprequest.ResponseHeader
        self.page_class_prev = httprequest.pagetemplate.Page
        self.cookie_name_prev = httprequest.session_cookie_name
        self.dispatch_method_prev = self.handler_class._dispatch
        httprequest.Response = Mock('Response_class')
        httprequest.Response.mock_returns = Mock('Response')
        httprequest.ResponseHeader = Mock('ResponseHeader_class')
        httprequest.ResponseHeader.mock_returns = Mock('ResponseHeader')
        httprequest.pagetemplate.Page = Mock('Page_class')
        httprequest.pagetemplate.Page.mock_returns = Mock('Page')
        httprequest.session_cookie_name = "TEST_session"
        mock_openid_server = Mock('openid_server')

        self.valid_requests = {
            'get-bogus': dict(
                request = Stub_Request("GET", "/bogus"),
            ),
            'get-root': dict(
                request = Stub_Request("GET", "/"),
            ),
            'no-cookie': dict(
                request = Stub_Request("GET", "/"),
            ),
            'unknown-cookie': dict(
                request = Stub_Request("GET", "/",
                    header = [
                        ("Cookie", "TEST_session=DECAFBAD"),
                    ],
                ),
                session_id = "DECAFBAD",
            ),
            'good-cookie': dict(
                identity_name = "fred",
                request = Stub_Request("GET", "/",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                    ],
                ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                ),
            ),
            'id-bogus': dict(
                identity_name = "bogus",
                request = Stub_Request("GET", "/id/bogus"),
            ),
            'id-fred': dict(
                identity_name = "fred",
                request = Stub_Request("GET", "/id/fred"),
            ),
            'logout': dict(
                request = Stub_Request("GET", "/logout"),
            ),
            'login': dict(
                request = Stub_Request("GET", "/login"),
            ),
            'nobutton-login': dict(
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                    ),
                ),
            ),
            'cancel-login': dict(
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                        cancel="Cancel",
                    ),
                ),
            ),
            'login-bogus': dict(
                identity_name = "bogus",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                        submit="Sign in",
                    ),
                ),
            ),
            'login-fred-wrong': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="fred",
                        password="password23",
                        submit="Sign in",
                    ),
                ),
            ),
            'login-fred-okay': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="fred",
                        password="password1",
                        submit="Sign in",
                    ),
                ),
            ),
            'openid-no-query': dict(
                request = Stub_Request("GET", "/openidserver"),
            ),
            'openid-bogus-query': dict(
                request = Stub_Request("GET", "/openidserver",
                    query = {
                        "foo.bar": "spam",
                        "flim.flam": "",
                        "wibble.wobble": "eggs",
                    },
                ),
            ),
            'openid-query-associate': dict(
                request = Stub_Request("GET", "/openidserver",
                    query = {
                        "openid.mode": "associate",
                        "openid.session_type": "",
                    },
                ),
            ),
            'openid-query-checkid_immediate-no-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
            ),
            'openid-query-checkid_setup-no-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
            ),
            'openid-query-checkid_immediate-other-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-bill"),
                    ],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
                session = dict(
                    session_id = "DEADBEEF-bill",
                    username = "bill",
                ),
            ),
            'openid-query-checkid_setup-other-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-bill"),
                    ],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
                session = dict(
                    session_id = "DEADBEEF-bill",
                    username = "bill",
                ),
            ),
            'openid-query-checkid_immediate-right-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                    ],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                ),
            ),
            'openid-query-checkid_setup-right-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                    ],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                    },
                ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                ),
            ),
            'openid-cancel-login': dict(
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                    ],
                    query = dict(
                        username = "bogus",
                        password = "bogus",
                        cancel = "Cancel",
                    ),
                ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                        ),
                    ),
                ),
            ),
            'openid-login-bill-other': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                    ],
                    query = dict(
                        username="bill",
                        password="secret1",
                        submit="Sign in",
                    ),
                ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                        ),
                    ),
                ),
            ),
            'openid-login-fred-okay': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                    ],
                    query = dict(
                        username="fred",
                        password="password1",
                        submit="Sign in",
                    ),
                ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                        ),
                    ),
                ),
            ),
        }

        logging.basicConfig(stream=self.stdout_test)

        for key, params in self.valid_requests.items():
            args = params.get('args')
            request = params['request']
            opts = Stub_OptionValues(dict(
                host = "example.org",
                port = 0,
                root_url = "http://example.org:0/",
            ))
            gracie_server = Stub_GracieServer(opts)
            sess_manager = gracie_server.sess_manager
            session = params.get('session', dict())
            sess_manager.create_session(session)
            server = params.setdefault('server',
                gracie_server.http_server
            )
            mock_openid_request = self._make_mock_openid_request(
                request.query
            )
            mock_openid_server = self._make_mock_openid_server(
                mock_openid_request
            )
            server.gracie_server.openid_server = mock_openid_server
            if not args:
                args = dict(
                    request = request.connection(),
                    client_address = (opts.host, opts.port),
                    server = server,
                )
            params['args'] = args

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_requests
        )

        version = Stub_GracieServer.version
        self.expect_server_version = "Gracie/%(version)s" % locals()
        python_version = sys.version.split()[0]
        self.expect_sys_version = "Python/%(python_version)s" % locals()

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        httprequest.Response = self.response_class_prev
        httprequest.ResponseHeader = self.response_header_class_prev
        httprequest.pagetemplate.Page = self.page_class_prev
        httprequest.session_cookie_name = self.cookie_name_prev
        self.handler_class._dispatch = self.dispatch_method_prev

    def _make_mock_openid_request(self, http_query):
        """ Make a mock OpenIDRequest for a given HTTP query """
        openid_request = Mock('OpenIDRequest')
        keys = ('mode', 'identity', 'trust_root', 'return_to')
        query_key_prefix = 'openid.'
        for key in keys:
            query_key = '%(query_key_prefix)s%(key)s' % locals()
            setattr(openid_request, key, http_query.get(query_key))
        openid_request.immediate = (
            openid_request.mode in ['checkid_immediate']
        )
        openid_request.answer.mock_returns = Stub_OpenIDResponse()

        return openid_request

    def _make_mock_openid_server(self, openid_request):
        """ Make a mock OpenIDServer for a given HTTP query """
        def stub_sign(obj):
            return obj

        openid_server = Mock('openid_server')

        if openid_request.mode:
            openid_server.decodeRequest.mock_returns = \
                openid_request
            openid_server.handleRequest.mock_returns = \
                Stub_OpenIDResponse()
            openid_server.encodeResponse.mock_returns = \
                Stub_OpenIDWebResponse()
        openid_server.signatory.sign = stub_sign

        return openid_server

    def test_instantiate(self):
        """ New HTTPRequestHandler instance should be created """
        for key, params in self.iterate_params():
            instance = self.handler_class(**params['args'])
            self.failUnless(instance is not None)

    def test_server_as_specified(self):
        """ HTTPRequestHandler should have specified server attribute """
        for key, params in self.iterate_params():
            instance = self.handler_class(**params['args'])
            server = params['server']
            self.failUnlessEqual(server, instance.server)

    def test_server_version_as_specified(self):
        """ HTTPRequestHandler should report module version """
        for key, params in self.iterate_params():
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(
                self.expect_server_version, instance.server_version
            )

    def test_version_string_as_specified(self):
        """ HTTPRequestHandler should report expected version string """
        params = self.valid_requests['get-bogus']
        instance = self.handler_class(**params['args'])
        expect_sys_version = self.expect_sys_version
        expect_server_version = self.expect_server_version
        expect_version_string = (
            "%(expect_server_version)s %(expect_sys_version)s"
            % locals() )
        version_string = instance.version_string()
        self.failUnlessEqual(expect_version_string, version_string)

    def test_command_from_request(self):
        """ Request command attribute should come from request text """
        for key, params in self.iterate_params():
            request = params['request']
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(request.method, instance.command)

    def test_path_from_request(self):
        """ Request path attribute should come from request text """
        for key, params in self.iterate_params():
            request = params['request']
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(request.path, instance.path)

    def test_internal_error_sends_server_error_response(self):
        """ When an error condition is raised, should send Server Error """
        class TestingError(StandardError):
            pass
        def raise_Error(_):
            raise TestingError("Testing error")
        self.handler_class._dispatch = raise_Error

        params = self.valid_requests['get-root']
        try:
            instance = self.handler_class(**params['args'])
        except TestingError:
            pass
        expect_stdout = """\
            Called ResponseHeader_class(500)
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_request_with_no_cookie_response_creates_session(self):
        """ With no session cookie, response should create new session """
        params = self.valid_requests['no-cookie']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            ...
            Called Response.header.fields.append(('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_request_with_unknown_cookie_creates_new_session(self):
        """ With unknown username, response should create new session """
        params = self.valid_requests['unknown-cookie']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            ...
            Called Response.header.fields.append(('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_request_with_good_cookie_response_logged_in(self):
        """ With good session cookie, response should send Logged In """
        params = self.valid_requests['good-cookie']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            ...
            Called Response.header.fields.append(
                ('Set-Cookie', 'TEST_session=DEADBEEF-%(identity_name)s'))
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_root_sends_ok_response(self):
        """ Request to GET root document should send OK response """
        params = self.valid_requests['get-root']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_bogus_url_sends_not_found_response(self):
        """ Request to GET unknown URL should send Not Found response """
        params = self.valid_requests['get-bogus']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(404)
            Called Page_class('...')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_bogus_identity_sends_not_found_response(self):
        """ Request to GET unknown user should send Not Found response """
        params = self.valid_requests['id-bogus']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(404)
            Called Page_class('...')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_known_identity_sends_view_user_response(self):
        """ Request to GET known user should send view user response """
        params = self.valid_requests['id-fred']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            Called Page_class('...')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def get_logout_creates_new_session_and_redirects(self):
        """ Request to logout should create new session and logout """
        params = self.valid_requests['logout']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(302)
            Called Response.header.fields.append('Location', ...)
            Called Response.header.fields.append(('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_login_sends_login_form_response(self):
        """ Request to GET login should send login form as response """
        params = self.valid_requests['login']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            Called Page_class('Login')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_nobutton_login_sends_not_found_response(self):
        """ POST login with no button should send Not Found response """
        params = self.valid_requests['nobutton-login']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(404)
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_login_cancel_no_openid_redirects_to_root(self):
        """ Login cancel with no OpenID should redirect to root """
        params = self.valid_requests['cancel-login']
        instance = self.handler_class(**params['args'])
        root_url = params['server'].gracie_server.opts.root_url
        expect_stdout = """\
            Called ResponseHeader_class(302)
            Called ResponseHeader.fields.append(('Location', %(root_url)r))
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_openid_login_cancel_redirects_to_openid_url(self):
        """ Login cancel with OpenID should redirect to OpenID URL """
        params = self.valid_requests['openid-cancel-login']
        openid_request = params['session']['last_openid_request']
        instance = self.handler_class(**params['args'])
        return_url = openid_request.answer(False).encodeToURL()
        expect_stdout = """\
            Called ResponseHeader_class(302)
            Called ResponseHeader.fields.append(
                ('Location', %(return_url)r))
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_login_bogus_user_sends_failure_response(self):
        """ POST login with bogus user should send failure response """
        params = self.valid_requests['login-bogus']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            Called Page_class('Login Failed')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_login_wrong_password_sends_failure_response(self):
        """ POST login with wrong password should send failure response """
        params = self.valid_requests['login-fred-wrong']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            Called Page_class('Login Failed')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_login_auth_correct_no_openid_redirects_to_root(self):
        """ Login with no OpenID, correct details should redirect to root """
        params = self.valid_requests['login-fred-okay']
        identity_name = params['identity_name']
        root_url = params['server'].gracie_server.opts.root_url
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(302)
            Called ResponseHeader.fields.append(('Location', %(root_url)r))
            ...
            Called Response.send_to_handler(
                ...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_openid_login_auth_correct_redirects_to_openid_url(self):
        """ Login correct with OpenID should redirect to OpenID URL """
        params = self.valid_requests['openid-login-fred-okay']
        openid_request = params['session']['last_openid_request']
        instance = self.handler_class(**params['args'])
        return_url = openid_request.answer(True).encodeToURL()
        expect_stdout = """\
            Called ResponseHeader_class(302)
            Called ResponseHeader.fields.append(
                ('Location', %(return_url)r))
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_post_openid_login_auth_other_sends_wrong_auth(self):
        """ Login wrong auth with OpenID should send Auth Required """
        params = self.valid_requests['openid-login-bill-other']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called ResponseHeader_class(200)
            Called Page_class('Authentication Required')
            ...
            Called Response.send_to_handler(...)
            """ % locals()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_server_no_query_sends_about_site_response(self):
        """ GET to server without query should send About page """
        params = self.valid_requests['openid-no-query']
        instance = self.handler_class(**params['args'])
        expect_stdout = """\
            Called openid_server.decodeRequest(...)
            Called ResponseHeader_class(200)
            Called Page_class('About this site')
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_server_bogus_query_sends_server_error_response(self):
        """ GET to server with bogus query should send Server Error """
        def raise_ProtocolError(_):
            raise Stub_OpenIDError("Testing error")

        params = self.valid_requests['openid-bogus-query']
        args = params['args']
        server = args['server']
        server.gracie_server.openid_server.decodeRequest = \
            raise_ProtocolError
        try:
            instance = self.handler_class(**args)
        except Stub_OpenIDError:
            pass
        expect_stdout = """\
            Called ResponseHeader_class(500)
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_get_server_assoc_query_delegates_to_openid(self):
        """ OpenID associate query should be passed to openid server """
        params = self.valid_requests['openid-query-associate']
        args = params['args']
        server = args['server']
        instance = self.handler_class(**args)
        expect_stdout = """\
            Called openid_server.decodeRequest(...)
            Called openid_server.handleRequest(...)
            Called openid_server.encodeResponse(...)
            Called ResponseHeader_class(200)
            Called ResponseHeader.fields.append(('openid', 'yes'))
            Called Response_class(..., 'OpenID response')
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_checkid_immediate_no_session_returns_failure(self):
        """ OpenID check_immediate with no session should reject """
        params_key = 'openid-query-checkid_immediate-no-session'
        params = self.valid_requests[params_key]
        args = params['args']
        server = args['server']
        instance = self.handler_class(**args)
        expect_stdout = """\
            Called openid_server.decodeRequest(...)
            Called OpenIDRequest.answer(False, ...)
            Called openid_server.encodeResponse(...)
            ...
            Called Response.send_to_handler(...)
            """
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_checkid_setup_wrong_session_returns_wrong_auth(self):
        """ OpenID checkid_setup with wrong session should request login """
        for params_key in [
            'openid-query-checkid_setup-no-session',
            'openid-query-checkid_setup-other-session',
        ]:
            params = self.valid_requests[params_key]
            args = params['args']
            server = args['server']
            instance = self.handler_class(**args)
            expect_stdout = """\
                Called openid_server.decodeRequest(...)
                Called ResponseHeader_class(200)
                Called Page_class('Authentication Required')
                ...
                Called Response.send_to_handler(...)
                """
            self.failUnlessOutputCheckerMatch(
                expect_stdout, self.stdout_test.getvalue()
            )

    def test_checkid_with_session_returns_success(self):
        """ OpenID checkid with right session should succeed """
        for params_key in [
            'openid-query-checkid_immediate-right-session',
            'openid-query-checkid_setup-right-session',
        ]:
            params = self.valid_requests[params_key]
            args = params['args']
            server = args['server']
            instance = self.handler_class(**args)
            expect_stdout = """\
                Called openid_server.decodeRequest(...)
                Called OpenIDRequest.answer(True)
                Called openid_server.encodeResponse(...)
                ...
                Called Response.send_to_handler(...)
                """
            self.failUnlessOutputCheckerMatch(
                expect_stdout, self.stdout_test.getvalue()
            )


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)
